<?php

namespace Moneybird;

require_once dirname(__FILE__) . '/../ApiConnector.php';

/**
 * Test class for Estimate.
 * Generated by PHPUnit on 2012-07-15 at 12:56:35.
 */
class EstimateTest extends \PHPUnit_Framework_TestCase {

	/**
	 * @var Estimate
	 */
	protected $object;
	
	/**
	 * @var int
	 */
	protected static $estimateId;
	
	/**
	 * @var string
	 */
	protected static $estimateNumber;
	
	protected static $config;
	
	protected static $contact;
	
	protected static $taxRateId;
	
	/**
	 * @var Contact_Service
	 */
	protected $service;
	
	/**
	 * @var ApiConnector
	 */
	protected $apiConnector;
	
	public static function setUpBeforeClass() {
		include ('config.php');
		
        self::$estimateNumber = null;
		self::$estimateId = null;
		
		self::$config = $config;
		$transport = getTransport($config);	
		$mapper = new XmlMapper();
		$connector = new ApiConnector($config['clientname'], $transport, $mapper);
		self::$contact = $connector->getService('Contact')->getById($config['testcontact']);
		
		$rates = $connector->getService('TaxRate')->getAll('sales');
		self::$taxRateId = current($rates)->id;
    }

	/**
	 * Sets up the fixture, for example, opens a network connection.
	 * This method is called before a test is executed.
	 */
	protected function setUp() {
		include ('config.php');
		
		$transport = getTransport($config);
		$mapper = new XmlMapper();
		$this->apiConnector = new ApiConnector($config['clientname'], $transport, $mapper);
		$this->service = $this->apiConnector->getService('Estimate');
		if (!is_null(self::$estimateId)) {
			$this->object = $this->service->getById(self::$estimateId);
		}
	}

	/**
	 * Tears down the fixture, for example, closes a network connection.
	 * This method is called after a test is executed.
	 */
	protected function tearDown() {
		
	}
	
	/**
	 * @covers Moneybird\Estimate::setContact
	 */
	public function testSetContact() {
		$this->assertInstanceOf('Moneybird\Contact', self::$contact);
		
		$estimate = new Estimate(array(), self::$contact);
		$this->assertEquals(self::$contact->address1, $estimate->address1);
				
		$estimate = new Estimate();
		$estimate->setContact(self::$contact);
		$this->assertEquals(self::$contact->address1, $estimate->address1);
	}
	
	/**
	 * @covers Moneybird\Estimate::save
	 */
	public function testSave() {
		$details = new Estimate_Detail_Array();
		$details->append(new Estimate_Detail(array(
			'amount' => 5, 
			'description' => 'My estimate line',
			'price' => 20,
			'taxRateId' => self::$taxRateId,
		)));
		$details->append(new Estimate_Detail(array(
			'amount' => 1, 
			'description' => 'My second estimate line',
			'price' => 12,
			'taxRateId' => self::$taxRateId,
		)));
		
		$estimate = new Estimate(array(
			'poNumber' => 'PO Number',
			'details' => $details,
			'lastname' => 'Custom lastname', 
		), self::$contact);
		
		$estimate->save($this->service);
		$this->assertInstanceOf('Moneybird\Estimate', $estimate);
		self::$estimateId = $estimate->id;
		$this->assertNotNull(self::$estimateId);
		$this->assertGreaterThan(0, self::$estimateId);
		
		$estimate->details[0]->setDeleted();
		$estimate->save($this->service);
		$this->assertEquals(1, count($estimate->details));
	}
	
	/**
	 * @covers Moneybird\Estimate_Service::getById
	 */
	public function testGetById() {
		$estimate = $this->service->getById(self::$estimateId);
		$this->assertInstanceOf('Moneybird\Estimate', $estimate);
		$this->assertEquals(self::$estimateId, $estimate->id);
	}

	/**
	 * @covers Moneybird\Estimate_Service::getAll
	 */
	public function testGetAll() {
		$estimates = $this->service->getAll('draft');
		$this->assertInstanceOf('Moneybird\Estimate_Array', $estimates);
		$this->assertGreaterThan(0, count($estimates), 'No estimates found');
		
		$estimates = self::$contact->getEstimates($this->service);
		$this->assertGreaterThan(0, count($estimates), 'No estimates found');
		foreach ($estimates as $estimate) {
			$this->assertEquals(self::$contact->id, $estimate->contactId);
		}
	}

	/**
	 * @covers Moneybird\Estimate::getPdf
	 */
	public function testGetPdf() {
		$this->setExpectedException('Moneybird\InvalidStateException');
		$this->object->getPdf($this->service);
	}

	/**
	 * @covers Moneybird\Estimate::send
	 */
	public function testSend() {
		$this->object->markAsSent($this->service);
		self::$estimateNumber = $this->object->estimateId;
		$this->assertNotNull(self::$estimateNumber);
	}

	/**
	 * @covers Moneybird\Estimate::markAsSent
	 */
	public function testMarkAsSent() {
		// Shorthand method not tested
	}
	
	/**
	 * @covers Moneybird\Estimate::delete
	 */
	public function testDelete() {
		$this->object->delete($this->service);
		$this->setExpectedException('Moneybird\NotFoundException');
		$this->service->getById(self::$estimateId);
	}
	 /* 
	 */

}

?>
